/* 
 * $Id: vscan-sophos.c,v 1.6 2001/12/02 16:39:16 reniar Exp $
 * 
 * virusscanning VFS module for samba.  Log infected files via syslog
 * facility and block access using SAVI.
 *
 * Copyright (C) Rainer Link, SuSE GmbH <link@suse.de>, 2001
 *                            OpenAntiVirus.org <rainer@openantivirus.org>
 *
 * general source code review by Thomas Biege, SuSE GmbH <thomas@suse.de>, 2001 
 * 
 * based on the audit VFS module by
 * Copyright (C) Tim Potter, 1999-2000
 *
 * based on a Sophie by
 * Copyright (C) Vanja Hrustic, 2001
 *
 * Credits to
 * - Dave Collier-Brown for his VFS tutorial (http://www.geocities.com/orville_torpid/papers/vfs_tutorial.html)
 * - REYNAUD Jean-Samuel for helping me to solve some general Samba VFS issues at the first place
 * - Simon Harrison for his solution without Samba VFS (http://www.smh.uklinux.net/linux/sophos.html)
 * - the whole Samba Team :)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <includes.h>
#include <vfs.h>

#include "vscan-sophos.h"




/* module version */
static const char module_id[]="$Revision: 1.6 $";

extern bool savi_ok;


#if SAMBA_VERSION_MAJOR!=3

/* Function prototypes */

int vscan_connect(struct connection_struct *conn, char *svc, char *user);
void vscan_disconnect(struct connection_struct *conn);

int vscan_open(struct connection_struct *conn, char *fname, int flags, mode_t mode);

#endif

/* VFS operations */

#if SAMBA_VERSION_MAJOR!=3
extern struct vfs_ops default_vfs_ops;   /* For passthrough operation */
#else
static struct vfs_ops default_vfs_ops;   /* For passthrough operation */
#endif

#if SAMBA_VERSION_MAJOR!=3 
struct vfs_ops vscan_ops = {
    
	/* Disk operations */

	vscan_connect,
	vscan_disconnect,
	NULL,                     /* disk free */

	/* Directory operations */

	NULL,
	NULL,                     /* readdir */
	NULL,
	NULL,
	NULL,                     /* closedir */

	/* File operations */

	vscan_open,
	NULL,
	NULL,                     /* read  */
	NULL,                     /* write */
	NULL,                     /* lseek */
	NULL,
	NULL,                     /* fsync */
	NULL,                     /* stat  */
	NULL,                     /* fstat */
	NULL,                     /* lstat */
	NULL,
	NULL,
	NULL,                     /* chown */
	NULL,                     /* chdir */
	NULL,                     /* getwd */
	NULL,                     /* utime */
	NULL,                     /* ftruncate */
	NULL,                     /* lock */
	NULL,                     /* fget_nt_acl */
	NULL,                     /* get_nt_acl */
	NULL,                     /* fset_nt_acl */
	NULL                      /* set_nt_acl */
};
#endif


/* Implementation of vfs_ops.  */

#if SAMBA_VERSION_MAJOR==3
static int vscan_connect(struct connection_struct *conn, const char *svc, const char *user)
#else
int vscan_connect(struct connection_struct *conn, char *svc, char *user)
#endif
{
	syslog(SYSLOG_PRIORITY, "connect to service %s by user %s\n", 
	       svc, user);

	vscan_sophos_init();

	#if SAMBA_VERSION_MAJOR!=3
	return default_vfs_ops.connect(conn, svc, user);
	#else
	return 0;	/* Success */
	#endif
}

#if SAMBA_VERSION_MAJOR==3
static void vscan_disconnect(struct connection_struct *conn)
#else
void vscan_disconnect(struct connection_struct *conn)
#endif
{
        if (savi_ok)
		vscan_sophos_end();

	syslog(SYSLOG_PRIORITY, "disconnected\n");

	#if SAMBA_VERSION_MAJOR!=3
	default_vfs_ops.disconnect(conn);
	#endif
}


#if SAMBA_VERSION_MAJOR==3
static int vscan_open(struct connection_struct *conn, const char *fname, int flags, mode_t mode)
#else
int vscan_open(struct connection_struct *conn, char *fname, int flags, mode_t mode)
#endif
{

	int retval;
	SMB_STRUCT_STAT stat_buf;


        if ( (stat(fname, &stat_buf)) != 0 )    /* an error occured */ 
		return default_vfs_ops.open(conn, fname, flags, mode);
	else if ( S_ISDIR(stat_buf.st_mode) ) 	/* is it a directory? */
		 return default_vfs_ops.open(conn, fname, flags, mode);
	else if ( ( stat_buf.st_size > VSCAN_SOPHOS_MAX_SIZE ) && ( VSCAN_SOPHOS_MAX_SIZE > 0 ) ) /* file is too large */
		syslog(SYSLOG_PRIORITY, "File %s too large! Not scanned!", fname);
	else if ( savi_ok ) 
	{
		retval = vscan_sophos_scanfile(fname);
		if ( retval == 1 ) {
			errno = EACCES; 
			return -1;
		}
	}

	return default_vfs_ops.open(conn, fname, flags, mode);

}


/* VFS initialisation function.  Return initialised vfs_ops structure
   back to SAMBA. */

#if SAMBA_VERSION_MINOR==2
struct vfs_ops *vfs_init(int* Version, struct vfs_ops *ops)
#elif SAMBA_VERSION_MINOR==1
struct vfs_ops *vfs_module_init(int *vfs_version)
#elif SAMBA_VERSION_MAJOR==3
struct vfs_ops *vfs_init(int *vfs_version, struct vfs_ops *ops)
#else
struct vfs_ops *vfs_init(int *vfs_version)
#endif
{
        openlog("smbd_vscan_sophos", LOG_PID, SYSLOG_FACILITY);

        #if SAMBA_VERSION_MINOR==2
        *Version = SMB_VFS_INTERFACE_VERSION;
        syslog(SYSLOG_PRIORITY, "VFS_INIT: vscan_ops - %s loaded\n", module_id);
	#elif SAMBA_VERSION_MAJOR==3
	*vfs_version = SMB_VFS_INTERFACE_VERSION;
	syslog(SYSLOG_PRIORITY, "VFS_INIT: vscan_ops - %s loaded\n", module_id);
        #else
        *vfs_version = SMB_VFS_INTERFACE_VERSION;
        syslog(SYSLOG_PRIORITY, "VFS_INIT: &vscan_ops: 0x%8.8x - %s\n",
               &vscan_ops, module_id);
        #endif

	#if SAMBA_VERSION_MAJOR==3
	/* Save a copy of the default ops */

        default_vfs_ops = *ops;

        /* Override our ones */
	ops->connect = vscan_connect;
	ops->disconnect = vscan_disconnect;
	ops->open = vscan_open;
	return(ops); 
	#else
        return(&vscan_ops);
	#endif
}

